/*
 * University of Illinois Open Source License
 * Copyright 2010-2011 Luthey-Schulten Group,
 * All rights reserved.
 * 
 * Developed by: Luthey-Schulten Group
 * 			     University of Illinois at Urbana-Champaign
 * 			     http://www.scs.uiuc.edu/~schulten
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the Software), to deal with 
 * the Software without restriction, including without limitation the rights to 
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
 * of the Software, and to permit persons to whom the Software is furnished to 
 * do so, subject to the following conditions:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimers.
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimers in the documentation 
 * and/or other materials provided with the distribution.
 * 
 * - Neither the names of the Luthey-Schulten Group, University of Illinois at
 * Urbana-Champaign, nor the names of its contributors may be used to endorse or
 * promote products derived from this Software without specific prior written
 * permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR 
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS WITH THE SOFTWARE.
 *
 * Author(s): Elijah Roberts
 */

#ifndef LM_CME_TWOSTATEHILLSWITCH_H_
#define LM_CME_TWOSTATEHILLSWITCH_H_

#include <cmath>
#include <map>
#include <string>

using std::string;
using std::map;

namespace lm {
namespace cme {

class TwoStateHillSwitch: public GillespieDSolver
{
protected:
    struct KHillPropensityArgs
    {
        KHillPropensityArgs(uint si, uint xi, double kmin, double kmax, double x50, double h) :si(si),xi(xi),kmin(kmin),kmax(kmax),dk(kmax-kmin),x50ph(pow(x50,h)),h(h) {}
        ~KHillPropensityArgs() {}
        uint si;
        uint xi;
        double kmin;
        double kmax;
        double dk;
        double x50ph;
        double h;
    };

public:
    TwoStateHillSwitch();
    virtual ~TwoStateHillSwitch();
    virtual bool needsReactionModel() {return false;}
    virtual bool needsDiffusionModel()  {return false;}
    virtual void generateTrajectory();
    virtual void runConstantProtein();
    virtual void runGeometricProtein();
	
protected:
    static double kHillPropensity(double time, uint * speciesCounts, void * pargs);
    static double negativeKHillPropensity(double time, uint * speciesCounts, void * pargs);
};

}
}

#endif
